<?php # -*- coding: utf-8 -*-

/**
 *  @class Oak_Access
 *  Relaciona un usuario con una acción y un permiso.
 *
 *  @author       Manuel Angel Rubio Jimenez
 *  @version      0.2
 *
 */
abstract class Oak_Access {
    protected $grants = ""; /*!< Accesos para una entidad. */
    protected $user = ""; /*!< Usuario al que se otorga el acceso. */

    /**
     *  Comprueba que el usuario existe y tiene acceso al sistema.
     *
     *  @param user_name nombre del usuario para comprobación de acceso.
     *  @param user_pass clave del usuario para comprobación de acceso.
     *  @return verdadero si todo ha ido bien y falso en caso contrario.
     */
	public function login( $user_name, $user_pass ) {
		if ($this->checkLogin($user_name, $user_pass)) {
			$this->grants = $this->getUserGrants($user_name);
			return true;
		}
		return false;
	}

    /**
     *  Esta función será implementada para realizar la comprobación
	 *  del login.
     *
     *  @param user_name nombre del usuario para comprobación de acceso.
     *  @param user_pass clave del usuario para comprobación de acceso.
     *  @return verdadero si todo ha ido bien y falso en caso contrario.
     */
    abstract protected function checkLogin( $user_name, $user_pass );


    /**
     *  Toma los permisos de un usuario.
     *
     *  @param user_name nombre del usuario.
     *  @return array de hashes con los permisos: <em>action</em>,
     *          <em>permi_id</em>, <em>perm_type</em>.
     */
    abstract public function getUserGrants( $user_name );


    /**
     *  Almacena los permisos pasados para un usuario.
     *
     *  Se encarga de eliminar todos los permisos actuales y
     *  almacenar solo los pasados por parámetro.
     *
     *  @param user_name usuario al que modificar los permisos.
     *  @param grants permisos a almacenar.
     *  @return true si todo ha ido bien, false en caso contrario.
     */
    abstract public function setUserGrants( $user_name, $grants );


    /**
     *  Elimina los permisos (access) de un usuario dado.
     *
     *  @param user_name usuario del que eliminar los permisos.
     *  @return true si todo ha ido bien, false o PEAR_Error en caso contrario.
     */
    abstract public function removeGrants( $user_name );


    /**
     *  Toma la información de los módulos respecto a permisos.
     *
     *  Cada módulo tiene un fichero <em>grants.php</em> en su directorio
     *  que hace referencia a los posibles permisos adjudicables para ese
     *  módulo.
     *
     *  @return árbol con los permisos.
     */
    public function getGlobalGrants( $dir = "" ) {
        global $modules_dir;
        if ($dir == "") {
            $dir = $modules_dir;
        }
        $d = dir($dir);
        $rt = array ();
        while (false !== ($entrada = $d->read())) {
            if ($entrada == 'grants.php') {
                include "$dir/$entrada";
                $rt = array_merge_recursive($rt, $grants);
                unset($grants);
            } elseif ($entrada != "." and $entrada != ".." and $entrada != '' and is_dir("$dir/$entrada")) {
                $rt = array_merge_recursive($rt, $this->getGlobalGrants("$dir/$entrada"));
            }
        }
        $d->close();
        return $rt;
    }


    /**
     *  Se pasa una estructura grants (getGlobalGrants) y se
     *  retorna una estructura anidada (jerárquica).
     *
     *  @param grants dato de tipo getGlobalGrants.
     *  @return reorganización jerárquica de grants.
     */
    public function getHierarchicalGrants( $grants ) {
        $hg = array();
        foreach ($grants as $k => $v) {
            $data = explode("_", $k);
            $tmp =& $hg;
            while (count($data) > 0) {
                $elemento = array_shift($data);
                $tmp =& $tmp[$elemento];
            }
            foreach ($v as $d) {
                $k2 = str_replace("_", "__", $k);
                if ($post[$k2 . '__' . $d]) {
                    $gp[] = $k2 . '__' . $d;
                }
                $tmp[] = $d;
            }
        }
        return $hg;
    }


    /**
     *  Se pasa una estructura de grants (getGlobalGrants) y el POST
     *  y se retorna un array con todos los elementos que se han
     *  marcado de un árbol.
     *
     *  @param grants dato de tipo getGlobalGrants.
     *  @param post array POST.
     *  @return array que contiene los elementos marcados de un árbol.
     */
    public function getGrantsPost( $grants, $post ) {
        $gp = array();
        foreach ($grants as $k => $v) {
            foreach ($v as $d) {
                $k2 = str_replace("::", "__", $k);
                if ($post[$k2 . '__' . $d]) {
                    $gp[] = $k2 . '__' . $d;
                }
            }
        }
        return $gp;
    }


    /**
     *  Comprueba el acceso.
     *
     *  @param action comprueba el acceso para la acción indicada.
     */
    public function checkAccess( $action ) {
        global $default_action, $anonymous_actions;
        if (is_array($anonymous_actions) and in_array($action, $anonymous_actions)) {
            return true;
		}
        if ($action == $default_action or $action == "Oak::Logout") {
            return true;
		}
        return (isset($this->grants[$action]) and $this->grants[$action]);
    }


    /**
     *  Obtiene los accesos específicos para una acción.
     *
     *  @param action rescata los permisos para la acción indicada.
     */
    public function getAccess( $action ) {
        if (isset($this->grants[$action])) {
            return $this->grants[$action];
        }
        return false;
    }


    /**
     *  Toma el valor de un atributo del objeto de tipo Access.
     *
     *  @param key nombre de un atributo.
     *  @return valor del atributo.
     */
    public function get( $key ) {
        return $this->$key;
    }


    /**
     *  Establece el valor de un atributo del objeto de tipo Access.
     *
     *  @param key nombre del atributo.
     *  @param val valor a asignarle.
     */
    public function set( $key, $val ) {
        $this->$key = $val;
    }
}
