<?php # -*- coding: utf-8 -*-

/**
 *  @class Oak_Components_Form
 *  Componentes de formulario para crear plantillas basadas en la clase
 *  Templates.
 *
 *  @author       Manuel Angel Rubio Jimenez
 *  @version      0.2
 *
 */
class Oak_Components_Form extends Oak_Components_Table {

    private $cf_index = 0; /*!< Número de formulario a agregar. */

    /**
     *  Constructor que se encarga de inicializar la estructura básica
     *  de un tipo de plantilla <em>login</em>.
     *
     *  @param template la plantilla que se usará para la salida.
     *  @param title título que se presentará en la barra de tareas y la
     *               ventana del navegador.
     *  @param subtitle se presentará en la cabecera de la página.
     *  @param name nombre que aparecerá como logo.
     *  @param lang idioma que se empleará.
     */
    function __construct( &$template, $title = "", $subtitle = "", $name = "", $lang = "" ) {
        parent::__construct($template, $title, $subtitle, $name, $lang);
        $this->template->put("main/@type", "form");
    }


    /**
     *  Configura la etiqueta <em>forms</em> de la plantilla.
     *
     *  @param title título que se presentará en la cabecera.
     *  @param type tipo de formularios que se presentarán (joomla).
     *  @param name nombre del formulario (joomla).
     */
    function setForms( $title = "", $type = "", $name = "" ) {
        $this->template->put("main/forms/@type", $type);
        $this->template->put("main/forms/@name", $name);
        $this->template->put("main/forms/@title", $title);
    }


    /**
     *  Agrega un formulario al conjunto de formularios.
     *
     *  @param fieldsets array o retorno de la función <strong>fieldset</strong>.
     *  @param id identificador del formulario.
     *  @param name nombre del formulario.
     */
    function addForm( $fieldsets = null, $id = "", $name = "" ) {
        $i = $this->cf_index++;
        $this->template->put("main/forms/form[$i]", array (
            "@id" => $id,
            "@name" => $name
        ));
		if ($fieldsets) {
            $this->template->put("main/forms/form[$i]/fieldset", $fieldsets);
		}
    }


    /**
     *  Retorna una estructura de <em>fieldset</em>.
     *
     *  @param legend título que mostrará el <em>fieldset</em>.
     *  @param fields array o retorno de la función <strong>field</strong>.
     *  @param submits array o retorno de la función <strong>submit</strong>.
     *  @param msg mensaje de error.
     *  @param table la tabla a agregar.
     *  @return array para inclusión en <em>addForm</em> u array para agrupar varios.
     */
    function fieldset( $legend, $fields, $submits = "", $msg = "", $table = "" ) {
        $data = array (
            "@legend" => $legend,
            "field" => $fields
        );
        if ($table != '') {
            $data["table"] = $table;
        }
        if ($submits != '') {
            $data["submit"] = $submits;
        }
        if ($msg != '') {
            $data["msg"]["@value"] = $msg;
        }
        return $data;
    }


    /**
     *  Retorna una estructura de <em>field</em>.
     *
     *  @param type el tipo de campo que se presentará.
     *  @param name nombre del campo (como variable para POST).
     *  @param label etiqueta que será presentada antes del campo de entrada de datos.
     *  @param value valor por defecto.
     *  @param size tamaño en columnas o caracteres.
     *  @param tip texto de ayuda cuando se pase el ratón por encima.
     *  @param action para botones, la acción a realizar al ser pulsados.
     *  @param options array o retorno de la función <strong>option</strong>.
     */
    function field( $type, $name, $label = '', $value = '', $size = '', $tip = '', $action = '', $options = '' ) {
        $data = array (
            "@type" => $type,
            "@name" => $name,
        );
        if ($label != '') {
            $data["label"] = $label;
        }
        if ($value != '') {
            $data["@value"] = $value;
        }
        if ($size != '') {
            $data["@size"] = $size;
        }
        if ($tip != '') {
            $data["tip"] = $tip;
        }
        if ($action != '' and $type == 'button') {
            $data["@action"] = $action;
        }
        if ($options != '') {
            $data["option"] = $options;
        }
        return $data;
    }


    /**
     *  Retorna una estructura de <em>option</em> anidada para incluir en
     *  <em>field</em>.
     *
     *  @param grants los permisos en forma jerárquica (getHierarchicalGrants)
     *  @param value es equivalente a <em>value</em>
     *  @param prefix de uso interno. se usa para conformar el nombre del checkbox.
     *  @return array de options para incluir en field.
     */
    public function option_tree( &$grants, $value, $prefix = "") {
        $key = 0;
        $i = $j = 0;
        foreach ($grants as $k => $v) {
            if (!is_numeric($k)) {
                $grants[$key]['@id'] = $k;
                $grants[$key]['option'] = $v;
                $grants[$key]['@value'] = "$prefix$k";
                unset($grants[$k]);
                $r = $this->option_tree($grants[$key]['option'], $value, $prefix . $k . '__');
                if ($r["sel"] > 0) {
                    if ($r["total"] - $r["sel"] == 0) {
                        $grants[$key]['@checked'] = 2;
                        $i++;
                    } else {
                        $grants[$key]['@checked'] = 1;
                        $i += 0.5;
                    }
                }
                $key++;
                $j++;
            } elseif (!is_array($v)) {
                $grants[$k] = array();
                $grants[$k]['@id'] = $v;
                $grants[$k]['@value'] = "$prefix$v";
                if (in_array("$prefix$v", $value)) {
                    $grants[$k]['@checked'] = 2;
                    $i++;
                }
                $grants[$k]['.'] = $v;
                $j++;
            }
        }
        if ($prefix != "") {
            return array ("total" => $j, "sel" => $i);
        }
        return $grants;
    }


    /**
     *  Retorna una estructura de <em>submit</em>.
     *
     *  @param name nombre para el botón de <em>submit</em>.
     *  @return array para inclusión en <em>fieldset</em> u array para agrupar varios.
     */
    function submit( $name ) {
        return array ( "@name" => $name );
    }


    /**
     *  Retorna una estructura de <em>field</em> para el tipo <em>text</em>.
     *
     *  @param name nombre del campo (como variable para POST).
     *  @param label etiqueta que será presentada antes del campo de entrada de datos.
     *  @param value valor por defecto.
     *  @param size tamaño en columnas o caracteres.
     *  @param tip texto de ayuda cuando se pase el ratón por encima.
     */
    function fieldText( $name, $label, $value = '', $size = '', $tip = '' ) {
        return $this->field("text", $name, $label, $value, $size, $tip);
    }


    /**
     *  Retorna una estructura de <em>field</em> para el tipo <em>image</em>.
     *
     *  @param name nombre del campo (como variable para POST).
     *  @param label etiqueta que será presentada antes del campo de entrada de datos.
     *  @param value valor por defecto.
     *  @param tip texto de ayuda cuando se pase el ratón por encima.
     */
    function fieldImage( $name, $label, $value = '', $size = '', $tip = '' ) {
        return $this->field("image", $name, $label, $value, $size, $tip);
    }


    /**
     *  Retorna una estructura de <em>field</em> para el tipo <em>hidden</em>.
     *
     *  @param name nombre del campo (como variable para POST).
     *  @param value valor que contendrá el campo.
     */
    function fieldHidden( $name, $value ) {
        return array (
            "@type" => "hidden",
            "@name" => $name,
            "@value" => $value
        );
    }


    /**
     *  Retorna una estructura de <em>field</em> para el tipo <em>read-only</em>.
     *
     *  @param name nombre del campo (como variable para POST).
     *  @param label etiqueta que será presentada antes del campo de entrada de datos.
     *  @param value valor por defecto.
     *  @param size tamaño en columnas o caracteres.
     */
    function fieldReadOnly( $name, $label, $value, $size = '' ) {
        $data = array (
            "@name" => $name,
            "@type" => "read-only",
            "label" => $label,
            "@value" => $value
        );
        if ($size != '') {
            $data["@size"] = $size;
        }
        return $data;
    }

}
