<?php # -*- coding: utf-8 -*-

/**
 *  @class Oak_Components_Table
 *  Componentes de tabla para crear plantillas basadas en la clase
 *  Templates.
 *
 *  @author       Manuel Angel Rubio Jimenez
 *  @version      0.1
 *
 */
class Oak_Components_Table extends Oak_Components {

    private $ct_index = 0; /*!< Número de tabla a agregar. */

    /**
     *  Constructor que se encarga de inicializar la estructura básica
     *  de un tipo de plantilla <em>table</em>.
     *
     *  @param template la plantilla que se usará para la salida.
     *  @param title título que se presentará en la barra de tareas y la
     *               ventana del navegador.
     *  @param subtitle se presentará en la cabecera de la página.
     *  @param name nombre que aparecerá como logo.
     *  @param lang idioma que se empleará.
     */
    function __construct( &$template, $title = "", $subtitle = "", $name = "", $lang = "" ) {
        parent::__construct($template, $title, $subtitle, $name, $lang);
        $this->template->put("main/@type", "table");
    }


    /**
     *  Configura la etiqueta <em>table</em> de la plantilla.
     *
     *  @param title título que se presentará en la cabecera.
     *  @param type tipo de tabla que se presentará (joomla).
     *  @param cols número de columnas que se presentarán (para paginación).
     *  @param rows número de filas que se presentarán (para paginación).
     */
    function setTable( $title = "", $type = "", $cols = 0, $rows = 0 ) {
        $this->template->put("main/table/@type", $type);
        $this->template->put("main/table/@title", $title);
        $this->template->put("main/table/@rows", $rows);
        $this->template->put("main/table/@cols", $cols);
    }


    /**
     *  Agrega una tabla.
     *
     *  @param thead cabeceras de la tabla.
     *  @param trow datos de la tabla.
     *  @param pagination información sobre paginación.
     */
    function addTable( $thead, $trow, $pagination = "" ) {
        $i = $this->ct_index++;
        $this->template->put("main/table[$i]/thead", $thead);
        $this->template->put("main/table[$i]/trow", $trow);
        if ($pagination != '') {
            $this->template->put("main/table[$i]/pagination", $pagination);
        }
    }


    /**
     *  Retorna una estructura de <em>table</em>.
     *
     *  @param thead cabeceras de la tabla.
     *  @param trow datos de la tabla.
     *  @param pagination información sobre paginación.
     *  @return array para inclusión en formularios u otros arrays.
     */
    static function table( $thead, $trow, $pagination = "" ) {
        $data["thead"] = $thead;
        $data["trow"] = $trow;
        if ($pagination != '') {
            $data["pagination"] = $pagination;
        }
        return $data;
    }


    /**
     *  Retorna una estructura de <em>toolbar-button</em>.
     *
     *  @param id identificador del icono.
     *  @param name nombre que se presentará en pantalla.
     *  @param tip texto de ayuda cuando se pase el ratón por encima.
     *  @return array con la estructura de un <em>toolbar-button</em>.
     */
    function toolbar_button( $id, $name, $tip = "" ) {
        return array (
            "@id" => $id,
            "@name" => $name,
            "@tip" => $tip
        );
    }


    /**
     *  Retorna una estructura de <em>toolbar</em>.
     *
     *  @param toolbar_buttons elementos del mismo tipo.
     */
    function setToolbar( $toolbar_buttons ) {
        $this->template->put('main/toolbar/toolbar-button', $toolbar_buttons);
    }


    /**
     *  Retorna una estructura de <em>thead</em>.
     *
     *  @param name nombre que presentará la cabecera de la columna.
     *  @param type tipo de datos que tendrá la columna.
     *  @return array para inclusión en <em>addTable</em> u array para agrupar varios.
     */
    function thead( $name, $type ) {
        $types = array ( "id", "center", "string", "text", "number", "currency", "date", "link", "check" );
        if (in_array($type, $types)) {
            return array (
                "@name" => $name,
                "@type" => $type
            );
        }
        throw new Exception(sprintf(_("Tipo %s no válido."), $type));
    }


    /**
     *  Retorna una estructura de <em>trow</em>.
     *
     *  @param tdata conjunto de <em>tdata</em>.
     *  @return array para inclusión en <em>addTable</em>.
     */
    function trow( $tdata ) {
        return array ( "tdata" => $tdata );
    }


    /**
     *  Retorna una estructura de <em>tdata</em> para incluir en <em>trow</em>.
     *
     *  @param content contenido de la celda.
     *  @param href enlace en caso de que la celda sea del tipo <em>link</em>.
     *  @param target donde se cargará el enlace al pulsarse (<em>link</em>).
     *  @param style estilo que se puede aplicar a la celda (xtracable).
     *  @return array para inclusión en <em>tdata</em> u array para agrupar varios.
     */
    function tdata( $content, $href = "", $target = "", $style = "" ) {
        $data = array ( "." => $content );
        if ($href != "") {
            $data["@href"] = $href;
        }
        if ($target != "") {
            $data["@target"] = $target;
        }
        if ($style != "") {
            $data["@style"] = $style;
        }
        return $data;
    }


    /**
     *  Retorna una estructura de <em>tdata</em> para incluir en <em>trow</em>.
     *
     *  @param content contenido de la celda.
     *  @param href enlace en caso de que la celda sea del tipo <em>link</em>.
     *  @param target donde se cargará el enlace al pulsarse (<em>link</em>).
     *  @param style estilo que se puede aplicar a la celda (xtracable).
     *  @return array para inclusión en <em>tdata</em> u array para agrupar varios.
     */
    function tdataSelect( $name, $options, $javascript = "", $style = "" ) {
        $data = array ( "@name" => $name, "option" => $options );
        if ($javascript != "") {
            $data["javascript"] = $javascript;
        }
        if ($style != "") {
            $data["@style"] = $style;
        }
        return $data;
    }


    /**
     *  Retorna una estructura de <em>option</em> para incluir en
     *  <em>field</em> o <em>tdataSelect</em>.
     *
     *  @param id identificador de la selección.
     *  @param label texto que se presentará.
     *  @param children array con suboptions (para checktree).
     *  @return array para inclusión en <em>field</em> u array para agrupar varios.
     */
    function option( $id, $label, $children = '' ) {
        $data = array (
            "@id" => $id,
            "." => $label
        );
        if ($children != '') {
            $data['option'] = $children;
        }
        return $data;
    }


    /**
     *  Retorna una estructura de <em>pagination</em>.
     *
     *  @param page la página actual que se muestra.
     *  @param total el total de tuplas que hay.
     *  @param size total de tuplas que se muestran por página.
     *  @param page_field campo donde se guarda la página actual (normalmente un hidden)
     *  @param size_field campo donde se guarda el tamaño actual (normalmente un hidden)
     *  @return array para inclusión en <em>addTable</em> o <em>table</em>.
     */
    function pagination( $page, $total, $size, $page_field, $size_field ) {
        $pags = array ();
        for ($i=1; $i<=ceil($total/$size); $i++) {
            $pags[] = $i;
        }

        $data = array (
            '@pages' => join(',', $pags),
            '@page' => $page,
            '@form' => 'form',
            '@page_field' => $page_field,
            '@size_field' => $size_field
        );
        return $data;
    }


    /**
     *  Retorna una estructura de <em>button</em> para el tipo <em>pagination</em>.
     *
     *  @param label etiqueta que se mostrará sobre el botón.
     *  @param active si está o no activo.
     *  @return array para inclusión en <em>pagination<em>.
     */
    function button( $label, $active ) {
        return array ( "@label" => $label, "@active" => $active );
    }

}
