<?php # -*- coding: utf-8 -*-

/**
 *  @class Oak_Menu
 *  La clase menú se encarga de construir los menús para Template, se recogerán
 *  todos los ficheros menu.php que se encuentren bajo el directorio base de
 *  módulos y se construirá el menú con las opciones a las que tenga acceso
 *  cada usuario.
 *
 *  @author       Manuel Angel Rubio Jimenez
 *  @version      0.2
 *
 */
class Oak_Menu {

    private $menu;   /*!< los datos del menú. */
    private $sort;   /*!< variable que se usará para ordenar los arrays de menú en load. */
    private $cut;    /*!< para cada plantilla, el valor que hay que cortar (cut). */
    private $logout; /*!< muestra o no "Salir". */
    private $opt;    /*!< valores para "optimize" recogidos de menu.php. */

    /**
     *  Constructor. Inicializa el valor de menu.
     *
     *  @param access objeto de tipo Access con los permisos del usuario.
     */
    function __construct( &$access ) {
		if (($access instanceof Oak_Access) and file_exists("conf/menu.yml")) {
			$this->cut = array();
			$this->logout = array();
			$this->opt = array();
			$this->menu = Horde_Yaml::loadFile("conf/menu.yml");
			$this->checkMenu($access);
		}
    }


    /**
     *  Elimina las opciones no válidas, según el objeto Access pasado como
     *  parámetro.
     *
     *  @param access objeto de tipo Access con los permisos del usuario.
     *  @param menu parámetro de uso interno para recursividad.
     */
    private function checkMenu( &$access, &$menu = null ) {
        if ($menu === null) {
            $menu =& $this->menu;
        }
        foreach ($menu as $k => $v) {
            if (is_array($menu[$k])) {
                $this->checkMenu($access, $menu[$k]);
                if (count($menu[$k]) == 0) {
                    unset($menu[$k]);
                }
            } else {
                if (!$access->checkAccess($menu[$k])) {
                    unset($menu[$k]);
                }
            }
        }
    }


    /**
     *  Elimina las opciones simples, llevando la opción a la raíz.
     *
     *  @param menu parámetro de uso interno para recursividad.
     */
    private function optimize( &$menu = null ) {
        if ($menu === null) {
            $menu =& $this->menu;
        }
        if (is_array($menu) and count($menu) > 0) {
            foreach ($menu as $k => $v) {
                if (is_array($menu[$k])) {
                    if (count($menu[$k]) == 1) {
                        $m = each($menu[$k]);
                        if (!is_array($m["value"])) {
                            $menu[$k] = $menu[$k][$m["key"]];
                        } else {
                            $this->optimize($menu[$k]);
                        }
                    } else {
                        $this->optimize($menu[$k]);
                    }
                }
            }
        }
    }


    /**
     *  Vuelca en el objeto Template el array de Menu.
     *
     *  @param template el objeto de tipo Plantilla donde se volcará el menú.
     *  @param opt indica si se deben de suprimir las opciones simples escalándolas (true), o no (false).
     */
    public function setTemplate( &$template, $opt = true ) {
        $t = $template->getTemplate();
        if (isset($this->cut[$t])) {
            $this->cutter($this->cut[$t]);
        }
        if ($opt and (!isset($this->opt[$t]) or $this->opt[$t])) {
            $this->optimize();
        }
        if (isset($this->logout[$t]) and !$this->logout[$t]) {
            unset($this->menu[_("Salir")]);
        }
        $td = $this->menu2template($this->menu);
        $template->put("menu", $td);
    }


    /**
     *  Transforma la estructura de Menu en la estructura de Menu
     *  para Template.
     *
     *  @param menu array empleado de forma recursiva.
     *  @return la transformación de la rama pasada.
     */
    private function menu2template( $menu ) {
        if (is_array($menu)) {
            $td = array ();
            foreach ($menu as $k => $v) {
                $d = array();
                if (is_array($menu[$k])) {
                    $d["@type"] = "node";
                    $d["@name"] = $k;
                    $t = $this->menu2template($menu[$k]);
                    $d = array_merge($d, $t);
                } else {
                    $d["@type"] = "item";
                    $d["@name"] = $k;
                    $d["@link"] = $v;
                }
                $td["menu-item"][] = $d;
            }
            return $td;
        }
        return array ();
    }


    /**
     *  Elimina un menú superior para convertirlo en múltiples opciones
     *  en la raíz del menú.
     *
     *  @param item opción del menú a cortar.
     */
    public function cutter( $item ) {
        if (isset($this->menu[$item])) {
            $t = $this->menu;
            $x = $t[$item];
            unset($t[$item]);
            $this->menu = array_merge_recursive($t, $x);
        }
    }

}
