<?php # -*- coding: utf-8 -*-

/**
 *  @class Oak_Session_Local
 *  Almacenará la información de Sesión.
 *
 *  Esta clase es un contenedor de lo que se almacenará en la sesión
 *  del usuario. La sesión se puede definir la consecución de peticiones
 *  del usuario desde la petición inicial de login hasta la petición
 *  final de logout.
 *
 *  @author       Manuel Angel Rubio Jiménez
 *  @version      0.2
 *
 */
class Oak_Session_Local extends Oak_Session {
    private $file; /*!< en caso de que se abra un fichero, aquí se guardará su nombre. */

    /**
     *  Operación que se encarga de abrir la sesión y realimentar el
     *  contenedor con los datos de la misma.
     */
    public function __construct( $file = null ) {
        global $_SESSION;
        if ($file == null) {
            session_start();
            $this->session =& $_SESSION;
            $this->file = null;
        } else {
            // no se crea una sesión, sino que se abre una existente (solo lectura)
            $ruta = session_save_path();
            if (file_exists("$ruta/$file")) {
                $f = file_get_contents("$ruta/$file");
                $this->session = $this->decode($f);
                $this->file = $file;
            }
        }
    }


    /**
     *  Lista las sesiones que hay activas.
     *
     *  @return un array con el nombre de las sesiones activas.
     */
    public static function getList() {
        $ruta = session_save_path();
        $d = @opendir($ruta);
        if (!$d) {
            return new PEAR_Error("No se pudo acceder al directorio de sesiones.");
        }
        $sessions = array ();
        while ($f = readdir($d)) {
            if (ereg("sess_[a-f0-9]+", $f)) {
                $sessions[] = $f;
            }
        }
        closedir($d);
        return $sessions;
    }


    /**
     *  Elimina la sessión. Si se especifica el parámetro <em>file</em>,
     *  elimina una sesión que exista, no la actual.
     *
     *  @param file se especifica si se desea eliminar una sesión existente.
     */
    public function deleteSession( $file = null ) {
        if (empty($file) and empty($this->file)) {
            foreach ($this->session as $k => $v) {
                unset($this->session[$k]);
            }
            session_destroy();
        } else {
			if (empty($file)) {
				$file = $this->file;
			}
            $ruta = session_save_path();
            if (file_exists("$ruta/$file")) {
                unlink("$ruta/$file");
            }
		}
    }


    /**
     *  Decodifica la cadena que contiene un fichero de sesión.
     *
     *  @param str cadena a decodificar.
     *  @return array con el contenido de una sesión.
     */
    private function decode( $str ) {
        $str = (string)$str;
        $endptr = strlen($str);
        $p = 0;

        $serialized = '';
        $items = 0;
        $level = 0;

        while ($p < $endptr) {
            for ($q=$p; ($str[$q] != '|') and ($q < $endptr); $q++)
                ;

            if ($str[$p] == '!') {
                $p++;
                $has_value = false;
            } else {
                $has_value = true;
            }

            $name = substr($str, $p, $q - $p);
            $q++;

            $serialized .= 's:' . strlen($name) . ':"' . $name . '";';

            if ($has_value) {
                for (;;) {
                    $p = $q;
                    switch ($str[$q]) {
                        case 'N': /* null */
                        case 'b': /* boolean */
                        case 'i': /* integer */
                        case 'd': /* decimal */
                            for ($q = $p+1; ($q < $endptr) and ($str[$q] != ';'); $q++ )
                                ;
                            $q++;
                            $serialized .= substr($str, $p, $q - $p);
                            if ($level == 0)
                                break 2;
                            break;
                        case 'R': /* reference  */
                            $q+= 2;
                            for ($id = ''; ($q < $endptr) && ($str[$q] != ';'); $q++)
                                $id .= $str[$q];
                            $q++;
                            $serialized .= 'R:' . ($id + 1) . ';'; /* increment pointer because of outer array */
                            if ($level == 0)
                                break 2;
                            break;
                        case 's': /* string */
                            $q+=2;
                            for ($length=''; ($q < $endptr) && ($str[$q] != ':'); $q++)
                                $length .= $str[$q];
                            $q+=2;
                            $q+= (int)$length + 2;
                            $serialized .= substr($str, $p, $q - $p);
                            if ($level == 0)
                                break 2;
                            break;
                        case 'a': /* array */
                        case 'O': /* object */
                            for ($q = $p+1; ( ($q < $endptr) and ($str[$q] != '{') ); $q++)
                                ;
                            $q++;
                            $level++;
                            $serialized .= substr($str, $p, $q - $p);
                            break;
                        case '}': /* end of array|object */
                            $q++;
                            $serialized .= substr($str, $p, $q - $p);
                            if (--$level == 0)
                                break 2;
                            break;
                        default:
                            return false;
                    }
                }
            } else {
                $serialized .= 'N;';
                $q+= 2;
            }
            $items++;
            $p = $q;
        }
        return @unserialize( 'a:' . $items . ':{' . $serialized . '}' );
    }
}
