<?php # -*- coding: utf-8 -*-

/**
 *  @class Oak_Template
 *  Esta clase abstracta se ha desarrollado como nexo de unión para la
 *  salida de la información mediante un sistema unificado de plantillas.
 *
 *  @author       Manuel Angel Rubio Jimenez
 *  @version      0.2
 *
 */
abstract class Oak_Template {
    
    protected $template;
    protected $data;
    private $attrib;        /*!< El método __find usará esta variable para
                               determinar si a donde llega es una hoja o
                               no. */
    protected $xml_flavor;

    /**
     *  El constructor se encarga de configurar la plantilla, inicialmente.
     *
     *  La única misión del constructor que se ha designado en estos momentos,
     *  es inicializar la variable de plantilla a un valor por defecto (joomla)
     *  o al valor pasado como parámetro.
     *
     *  @param template especifica la plantilla XSLT a usar.
     */
	public function __construct( $template = "joomla" ) {
		$this->template = $template;
		$this->data = array();
	}
    
    /**
     *  Método privado que se encarga de localizar, mediante una ruta XPath,
     *  el contenido solicitado. Se retorna una referencia a la rama, u hoja
     *  que se solicita.
     *
     *  @param path ruta en formato XPath.
     *  @return referencia a la rama u hoja que se solicita.
     */
    private function &__find( $path ) {
        // cortamos la cadena en un array por las barras (/)
        $parts = explode("/", $path);
        if ($parts[0] == "main") {
            array_shift($parts);
        }
        // enlazamos en $t por referencia el array "data"
        $t = &$this->data;
        $this->attrib = false;
        foreach ($parts as $p) {
            if (preg_match("/(.+)\[([0-9]+)\]$/", $p, $a)) {
                $t = &$t[$a[1]][$a[2]];
                $this->attrib = false;
            } else {
                // recorremos toda la ruta enlazando en cada parte
                if (preg_match("/^@(.+)/", $p, $a)) {
                    if ($this->xml_flavor) {
                        $t =& $t["_a"][$a[1]];
                    } else {
                        $t =& $t["_" . $a[1]];
                    }
                    $this->attrib = true;
                } else {
                    if ($p != ".") {
                        $t = &$t[$p];
                    }
                    $this->attrib = false;
                }
            }
        }
        return $t;
    }


    /**
     *  Toma información del árbol por una ruta dada.
     *
     *  Este método obtiene una ruta, separada por barras (/), que le servirá para
     *  elegir la "hoja" del árbol que se quiera tomar. Se ha intentado, dentro de
     *  lo posible, que el formato de la ruta mantenga compatibilidad con XPath.
     *
     *  @param path ruta para rescatar el nodo solicitado.
     *  @return el nodo solicitado, ya sea una "rama" (<em>array</em>) o una "hoja"
     *          (<em>string</em>).
     */
    function get( $path ) {
        return $this->__find($path);
    }


    /**
     *  Toma el árbol y lo retorna tal cual (útil para Components).
     *
     *  @return el árbol de datos para la plantilla.
     */
    function getData() {
        return $this->data;
    }


    /**
     *  Establece un valor para el árbol por una ruta dada.
     *
     *  Este método establece un valor, ya sea de "rama" (<em>array</em>) u "hoja"
     *  (<em>string</em>), a partir de una ruta pasada por parámetro. Se ha
     *  intentado, dentro de lo posible, que el formato de la ruta mantenga
     *  compatibilidad con XPath.
     *
     *  @param path ruta para establecer el valor.
     *  @param value valor que se establecerá en el nodo.
     */
    function put( $path, $value ) {
        // si es un array no se puede agregar tal cual...
        if (is_array($value)) {
            // se descompone y se ejecuta put sobre cada elemento
            foreach ($value as $k => $v) {
                $this->put("$path/$k", $v);
            }
        } else {
            // enlazamos en $t por referencia el array "data"
            if ($path == 'main') {
                $t =& $this->data;
            } else {
                $t =& $this->__find($path);
            }
            // asignamos el valor a la hoja a la que hemos llegado
            if (!$this->attrib and $this->xml_flavor) {
                // contenido
                $t["_c"] = $value;
            } else {
                // atributo o dato normal
                $t = $value;
            }
        }
    }

    /**
     *  Convierte los valores dados en la salida deseada.
     *
     *  @return texto HTML, generalmente.
     */
    abstract function render( $test = false );


    /**
     *  Fija la plantilla a usar.
     *
     *  Este método cambia el valor de la variable de clase <tt>template</tt>
     *  para cuando se use el método <tt>render</tt>, se haga la transformación
     *  usando esa plantilla.
     *
     *  @param template nombre de la plantilla.
     */
    public function setTemplate( $template ) {
        $this->template = $template;
    }

    /**
     *  Toma la plantilla que se usará.
     * 
     *  @return texto con el nombre de la plantilla.
     */
    public function getTemplate() {
        return $this->template;
    }

    /**
     *  Lista las posibles plantillas para usar.
     *
     *  @return un array con las posibles plantillas a usar.
     */
    public function listTemplate() {
        global $themes_dir;
        $templates = array();
        if (!empty($themes_dir)) {
            $d = opendir($themes_dir);
            while ($t = readdir($d)) {
                if (!ereg('^\.', $t)) {
                    $templates[] = $t;
                }
            }
            closedir($d);
        }
        return $templates;
    }
}
