<?php # -*- coding: utf-8 -*-

/**
 *  @class Util
 *  Esta clase contiene diversas funciones de uso general.
 *
 *  @author       Manuel Angel Rubio Jimenez
 *  @version      0.2
 *
 */
class Oak_Util {
    /**
     * Esta función clasifica y valida perfectamente todos los códigos fiscales que se usan en España.
     * Analiza una variable de 9 carácteres alfanuméricos. Los valores devueltos son:
     *  true = Si NIF, CIF o NIE son válidos
     *  false = Si NIF, CIF o NIE no son válidos
     *
     * La función cumple con todas las especificaciones de las leyes españolas:
     * - Decreto 2423/1975, de 25 de septiembre.
     * - Real Decreto 338/1990, de 9 de marzo.
     * - Real Decreto 155/1996, de 2 de febrero.
     * - Orden de 3 de julio de 1998, por la que se modifica el Anexo del Decreto 2423/1975.
     *
     * Que es, actualmente, todo lo aprobado respecto con los Códigos de Identificación Fiscal,
     * Número de Identificación Fiscal y Número de Identificación de Extranjeros.
     *
     * - Nota: dado que el algoritmo del dígito de control usado es secreto y jamás ha sido publicado 
     *   en fuentes oficiales del Estado se ha usado un algoritmo obtenido por ingeniería inversa que 
     *   ha logrado validar correctamente alrededor de un millón de CIFs, NIFs y NIEs en total. Con 
     *   lo que, presumiblemente, se puede asumir que concuerda con el algoritmo de validación 
     *   secreto original.
     *
     * @param documento el número del documento a validar (con letra incluida).
     * @param tipoDocumento el tipo de documento a validar (CIF, NIE o NIF).
     * @return 1 en caso de validación correcta, 0 en caso contrario.
     */
    static function valida_nif_cif_nie($documento, $tipoDocumento) {
        $documento=strtoupper($documento);
        if (!ereg('((^[A-Z]{1}[0-9]{7}[A-Z0-9]{1}$|^[T]{1}[A-Z0-9]{8}$)|^[0-9]{8}[A-Z]{1}$)',$documento)) {
            return false;
        }
        for ($i=0;$i<9;$i++) {
            $num[$i]=substr($documento,$i,1);
        }
        $suma=$num[2]+$num[4]+$num[6];
        for ($i=1;$i<8;$i+=2) {
            $suma+=substr((2*$num[$i]),0,1)+substr((2*$num[$i]),1,1);
        }
        $n=10-substr($suma,strlen($suma)-1,1);

        //cif
        if($tipoDocumento == 'CIF'){
            if (ereg('^[ABCDEFGHNPQS]{1}',$documento)) {
                return ($num[8]==chr(64+$n) or $num[8]==substr($n,strlen($n)-1,1));
            }
            if (ereg('^[KLM]{1}',$documento)) {
                return ($num[8]==chr(64+$n));
            }
        }

        //nie
        if($tipoDocumento == 'NIE'){
            if (ereg('^[TX]{1}',$documento)) {
                return ($num[8]==substr('TRWAGMYFPDXBNJZSQVHLCKE',substr(ereg_replace('X','0',$documento),0,8)%23,1) or ereg('^[T]{1}[A-Z0-9]{8}$',$documento));
            }
        }

        //nif
        if($tipoDocumento == 'NIF'){
            if (ereg('(^[0-9]{8}[A-Z]{1}$)',$documento)) {
                return ($num[8]==substr('TRWAGMYFPDXBNJZSQVHLCKE',substr($documento,0,8)%23,1));
            }
        }
        return false;
    }


    /**
     *  Función para validar el ICC-ID de las tarjetas de telefonía móvil.
     *
     *  Esta función está tomada de:
     *  http://planzero.org/code/bits/viewcode.php?src=ccardcheck.phps
     *
     *  @param iccid el código a validar de la tarjeta.
     *  @return verdadero en caso de que sea válido, falso en caso contrario.
     */
    static function validate_iccid( $iccid ) {
        $iccid = preg_replace("/\D|\s/", "", $iccid);  # strip any non-digits
        $iccidlength = strlen($iccid);
        $parity = $iccidlength % 2;
        $sum = 0;
        for ($i=0; $i<$iccidlength; $i++) {
            $digit = $iccid[$i];
            if ($i % 2 == $parity) {
                $digit *= 2;
            }
            if ($digit > 9) {
                $digit -= 9;
            }
            $sum += $digit;
        }
        return ($sum % 10 == 0);
    }


    /**
     * Función para validar e-mail.
     *
     * @param email E-mail a validar
     * @return  verdadero en caso de que se válido, falso en caso contrario.
     */
    static function validate_email ($email) {
        $pattern="/^([a-zA-Z0-9])+([\.a-zA-Z0-9_-])*@([a-zA-Z0-9_-])+(\.[a-zA-Z0-9_-]+)+$/";
        return (preg_match($pattern,$email) && strlen($email)<=100);
    }


    /**
     * Función para validar direcciones ip.
     *
     * @param email Ip a validar
     * @return  'ok' en caso de que sea valido, el error en caso contrario.
     */
    static function validate_ip ($ip) {
        if (preg_match("/([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})", $ip, $args)) {
            return
                ($args[1] > 0 or $args[1] <= 255) and
                ($args[2] >= 0 or $args[2] <= 255) and
                ($args[3] >= 0 or $args[3] <= 255) and
                ($args[4] >= 0 or $args[4] < 255);
        }
        return false;
    }

    /**
     *  Convierte un número entero (segundos) a formato 00:00:00.
     *
     *  @param segundos los segundos a convertir.
     */
    static function convertTime( $seg ) {
        $tmp = $seg;
        $min = $hor = 0;
        if ($seg >= 60) {
            $seg = $tmp % 60;
            $tmp = ($tmp - $seg) / 60;
            $min = $tmp;
            if ($min >= 60) {
                $min = $tmp % 60;
                $tmp = ($tmp - $min) / 60;
                $hor = $tmp;
            }
        }
        return sprintf("%02d:%02d:%02d", $hor, $min, $seg);
    }

	static function assert( $data ) {
		print "<pre>"; print_r($data); print "</pre>"; die;
	}
}
